/*
 * Copyright (c) 2017 Carmen Alvarez
 *
 * This file is part of Poet Assistant.
 *
 * Poet Assistant is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Poet Assistant is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Poet Assistant.  If not, see <http://www.gnu.org/licenses/>.
 */

package ca.rmen.android.poetassistant.main.dictionaries;

import android.app.Application;
import android.arch.lifecycle.AndroidViewModel;
import android.content.SharedPreferences;
import android.databinding.ObservableBoolean;
import android.databinding.ObservableField;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.support.v4.app.LoaderManager;
import android.support.v4.content.Loader;
import android.text.Spannable;
import android.text.SpannableStringBuilder;
import android.text.TextUtils;
import android.text.style.ImageSpan;
import android.util.Log;

import ca.rmen.android.poetassistant.Constants;
import ca.rmen.android.poetassistant.R;
import ca.rmen.android.poetassistant.compat.VectorCompat;
import ca.rmen.android.poetassistant.main.Tab;
import ca.rmen.android.poetassistant.settings.Settings;
import ca.rmen.android.poetassistant.settings.SettingsPrefs;

public class ResultListViewModel<T> extends AndroidViewModel {
    private static final String TAG = Constants.TAG + ResultListViewModel.class.getSimpleName();

    public final ObservableBoolean isDataAvailable = new ObservableBoolean();
    public final ObservableField<Settings.Layout> layout = new ObservableField<>();
    public final ObservableField<CharSequence> emptyText = new ObservableField<>();
    final ObservableBoolean showHeader = new ObservableBoolean();
    private final ObservableField<ResultListData<T>> data = new ObservableField<>();
    private ResultListAdapter<T> mAdapter;
    private Tab mTab;

    ResultListViewModel(Application application, Tab tab) {
        super(application);
        mTab = tab;
        emptyText.set(getNoQueryEmptyText());
        PreferenceManager.getDefaultSharedPreferences(application).registerOnSharedPreferenceChangeListener(mPrefsListener);
    }

    void setAdapter(ResultListAdapter<T> adapter) {
        mAdapter = adapter;
    }

    void share(Tab tab, String query, String filter) {
        Share.share(getApplication(), tab, query, filter, data.get().data);
    }

    String getUsedQueryWord() {
        return data.get() == null ? null : data.get().matchedWord;
    }

    private void updateDataAvailable() {
        isDataAvailable.set(mAdapter != null && mAdapter.getItemCount() > 0);
        isDataAvailable.notifyChange();
    }

    private void setData(ResultListData<T> loadedData) {
        Log.v(TAG, "setData " + loadedData);
        mAdapter.clear();
        if (loadedData != null) mAdapter.addAll(loadedData.data);
        data.set(loadedData);
        boolean hasQuery = data.get() != null && !TextUtils.isEmpty(data.get().matchedWord);
        if (!hasQuery) {
            emptyText.set(getNoQueryEmptyText());
        } else if (data.get().data != null) {
            emptyText.set(getNoResultsEmptyText(data.get().matchedWord));
        } else {
            emptyText.set(null);
        }
        showHeader.set(hasQuery);
        updateDataAvailable();
    }

    // If we have an empty list because the user didn't enter any search term,
    // we'll show a text to tell them to search.
    private CharSequence getNoQueryEmptyText() {
        String emptySearch = getApplication().getString(R.string.empty_list_without_query);
        ImageSpan imageSpan = VectorCompat.createVectorImageSpan(getApplication(), R.drawable.ic_action_search_dark);
        SpannableStringBuilder ssb = new SpannableStringBuilder(emptySearch);
        int iconIndex = emptySearch.indexOf("%s");
        ssb.setSpan(imageSpan, iconIndex, iconIndex + 2, Spannable.SPAN_INCLUSIVE_INCLUSIVE);
        return ssb;
    }

    // If the user entered a query and there are no matches, show the normal "no results" text.
    private CharSequence getNoResultsEmptyText(String query) {
        return ResultListFactory.getEmptyListText(getApplication(), mTab, query);
    }

    @Override
    protected void onCleared() {
        super.onCleared();
        PreferenceManager.getDefaultSharedPreferences(getApplication()).unregisterOnSharedPreferenceChangeListener(mPrefsListener);
    }

    private final SharedPreferences.OnSharedPreferenceChangeListener mPrefsListener = (sharedPreferences, key) -> {
        if (Settings.PREF_LAYOUT.equals(key)) {
            layout.set(Settings.getLayout(SettingsPrefs.get(getApplication())));
        }
    };

    // Should replace this with LiveData.
    final LoaderManager.LoaderCallbacks<ResultListData<T>> loaderCallbacks = new LoaderManager.LoaderCallbacks<ResultListData<T>>() {

        @Override
        public Loader<ResultListData<T>> onCreateLoader(int id, Bundle args) {
            Log.d(TAG, mTab + ": onCreateLoader() called with: " + "id = [" + id + "], args = [" + args + "]");

            String query = "";
            String filter = "";
            if (args != null && args.containsKey(ResultListFragment.EXTRA_QUERY)) {
                query = args.getString(ResultListFragment.EXTRA_QUERY);
                filter = args.getString(ResultListFragment.EXTRA_FILTER);
                setData(new ResultListData<>(query, null));
            } else {
                setData(null);
            }

            //noinspection unchecked
            return (Loader<ResultListData<T>>) ResultListFactory.createLoader(mTab, getApplication(), query, filter);
        }

        @Override
        public void onLoadFinished(Loader<ResultListData<T>> loader, ResultListData<T> data) {
            Log.d(TAG, mTab + ": onLoadFinished() called with: " + "loader = [" + loader + "], data = [" + data + "]");
            setData(data);
        }

        @Override
        public void onLoaderReset(Loader<ResultListData<T>> loader) {
            Log.d(TAG, mTab + ": onLoaderReset() called with: " + "loader = [" + loader + "]");
            setData(null);
        }
    };

}
